Stan Math Library: stan/math/prim/scal/prob/neg_binomial_2_cdf.hpp Source File

| Stan Math Library 2.10.0 reverse mode automatic differentiation

- Main Page
- Modules
- Namespaces
- Classes
- Files

- File List
- File Members

neg_binomial_2_cdf.hpp
Go to the documentation of this file.
1 #ifndef STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_2_CDF_HPP
2 #define STAN_MATH_PRIM_SCAL_PROB_NEG_BINOMIAL_2_CDF_HPP
3
4 #include <stan/math/prim/scal/err/check_consistent_sizes.hpp>
5 #include <stan/math/prim/scal/err/check_not_nan.hpp>
6 #include <stan/math/prim/scal/err/check_positive_finite.hpp>
7 #include <stan/math/prim/scal/fun/digamma.hpp>
8 #include <stan/math/prim/scal/fun/inc_beta.hpp>
9 #include <stan/math/prim/scal/fun/inc_beta_dda.hpp>
10 #include <stan/math/prim/scal/fun/inc_beta_ddb.hpp>
11 #include <stan/math/prim/scal/fun/inc_beta_ddz.hpp>
12 #include <stan/math/prim/scal/fun/value_of.hpp>
13 #include <stan/math/prim/scal/meta/is_constant_struct.hpp>
14 #include <stan/math/prim/scal/meta/length.hpp>
15 #include <stan/math/prim/scal/meta/partials_return_type.hpp>
16 #include <stan/math/prim/scal/meta/OperandsAndPartials.hpp>
17 #include <stan/math/prim/scal/meta/VectorBuilder.hpp>
18 #include <limits>
19
20 namespace stan {
21 namespace math {
22
23 template <typename T_n, typename T_location,
24 typename T_precision>
25 typename return_type<T_location, T_precision>::type
26 neg_binomial_2_cdf(const T_n& n,
27 const T_location& mu,
28 const T_precision& phi) {
29 static const char* function("stan::prob::neg_binomial_2_cdf");
30 typedef typename stan::partials_return_type<T_n, T_location,
31 T_precision>::type
32 T_partials_return;
33
34 using stan::math::check_positive_finite;
35 using stan::math::check_not_nan;
36 using stan::math::check_consistent_sizes;
37
38 T_partials_return P(1.0);
39 // check if any vectors are zero length
40 if (!(stan::length(n)
41 && stan::length(mu)
42 && stan::length(phi)))
43 return P;
44
45 // Validate arguments
46 check_positive_finite(function, "Location parameter", mu);
47 check_positive_finite(function, "Precision parameter", phi);
48 check_not_nan(function, "Random variable", n);
49 check_consistent_sizes(function,
50 "Random variable", n,
51 "Location parameter", mu,
52 "Precision Parameter", phi);
53
54 // Wrap arguments in vector views
55 VectorView<const T_n> n_vec(n);
56 VectorView<const T_location> mu_vec(mu);
57 VectorView<const T_precision> phi_vec(phi);
58 size_t size = max_size(n, mu, phi);
59
60 // Compute vectorized CDF and gradient
61 using stan::math::value_of;
62 using stan::math::inc_beta;
63 using stan::math::inc_beta_ddz;
64 using stan::math::inc_beta_dda;
65 using stan::math::digamma;
66
67 OperandsAndPartials<T_location, T_precision>
68 operands_and_partials(mu, phi);
69
70 // Explicit return for extreme values
71 // The gradients are technically ill-defined, but treated as zero
72 for (size_t i = 0; i < stan::length(n); i++) {
73 if (value_of(n_vec[i]) < 0)
74 return operands_and_partials.value(0.0);
75 }
76
77 // Cache a few expensive function calls if phi is a parameter
78 VectorBuilder<!is_constant_struct<T_precision>::value,
79 T_partials_return, T_precision>
80 digamma_phi_vec(stan::length(phi));
81
82 VectorBuilder<!is_constant_struct<T_precision>::value,
83 T_partials_return, T_precision>
84 digamma_sum_vec(stan::length(phi));
85
86 if (!is_constant_struct<T_precision>::value) {
87 for (size_t i = 0; i < stan::length(phi); i++) {
88 const T_partials_return n_dbl = value_of(n_vec[i]);
89 const T_partials_return phi_dbl = value_of(phi_vec[i]);
90
91 digamma_phi_vec[i] = digamma(phi_dbl);
92 digamma_sum_vec[i] = digamma(n_dbl + phi_dbl + 1);
93 }
94 }
95
96 for (size_t i = 0; i < size; i++) {
97 // Explicit results for extreme values
98 // The gradients are technically ill-defined, but treated as zero
99 if (value_of(n_vec[i]) == std::numeric_limits<int>::max())
100 return operands_and_partials.value(1.0);
101
102 const T_partials_return n_dbl = value_of(n_vec[i]);
103 const T_partials_return mu_dbl = value_of(mu_vec[i]);
104 const T_partials_return phi_dbl = value_of(phi_vec[i]);
105
106 const T_partials_return p_dbl = phi_dbl / (mu_dbl + phi_dbl);
107 const T_partials_return d_dbl = 1.0 / ((mu_dbl + phi_dbl)
108 * (mu_dbl + phi_dbl));
109
110 const T_partials_return P_i =
111 inc_beta(phi_dbl, n_dbl + 1.0, p_dbl);
112
113 P *= P_i;
114
115 if (!is_constant_struct<T_location>::value)
116 operands_and_partials.d_x1[i] +=
117 - inc_beta_ddz(phi_dbl, n_dbl + 1.0, p_dbl) * phi_dbl * d_dbl / P_i;
118
119 if (!is_constant_struct<T_precision>::value) {
120 operands_and_partials.d_x2[i]
121 += inc_beta_dda(phi_dbl, n_dbl + 1, p_dbl,
122 digamma_phi_vec[i],
123 digamma_sum_vec[i]) / P_i
124 + inc_beta_ddz(phi_dbl, n_dbl + 1.0, p_dbl)
125 * mu_dbl * d_dbl / P_i;
126 }
127 }
128
129 if (!is_constant_struct<T_location>::value) {
130 for (size_t i = 0; i < stan::length(mu); ++i)
131 operands_and_partials.d_x1[i] *= P;
132 }
133
134 if (!is_constant_struct<T_precision>::value) {
135 for (size_t i = 0; i < stan::length(phi); ++i)
136 operands_and_partials.d_x2[i] *= P;
137 }
138
139 return operands_and_partials.value(P);
140 }
141
142 }
143 }
144 #endif
check_not_nan.hpp
stan::math::OperandsAndPartials::d_x2
VectorView< T_return_type, false, true > d_x2
Definition: OperandsAndPartials.hpp:40
stan::math::neg_binomial_2_cdf
return_type< T_location, T_precision >::type neg_binomial_2_cdf(const T_n &n, const T_location &mu, const T_precision &phi)
Definition: neg_binomial_2_cdf.hpp:26
stan::math::check_not_nan
bool check_not_nan(const char *function, const char *name, const T_y &y)
Return true if y is not NaN.
Definition: check_not_nan.hpp:63
check_positive_finite.hpp
stan
Definition: log_sum_exp.hpp:8
stan::math::value_of
T value_of(const fvar< T > &v)
Return the value of the specified variable.
Definition: value_of.hpp:16
stan::math::inc_beta_dda
T inc_beta_dda(T a, T b, T z, T digamma_a, T digamma_ab)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to a.
Definition: inc_beta_dda.hpp:39
inc_beta_ddz.hpp
digamma.hpp
stan::math::OperandsAndPartials::value
T_return_type value(double value)
Returns a T_return_type with the value specified with the partial derivatves.
Definition: OperandsAndPartials.hpp:68
stan::length
size_t length(const std::vector< T > &x)
Definition: length.hpp:10
is_constant_struct.hpp
stan::math::inc_beta_ddz
T inc_beta_ddz(T a, T b, T z)
Returns the partial derivative of the regularized incomplete beta function, I_{z}(a, b) with respect to z.
Definition: inc_beta_ddz.hpp:27
inc_beta_ddb.hpp
stan::is_constant_struct
Metaprogram to determine if a type has a base scalar type that can be assigned to type double...
Definition: is_constant_struct.hpp:13
stan::math::inc_beta
fvar< T > inc_beta(const fvar< T > &a, const fvar< T > &b, const fvar< T > &x)
Definition: inc_beta.hpp:20
stan::partials_return_type
Definition: partials_return_type.hpp:16
stan::math::OperandsAndPartials
This class builds partial derivatives with respect to a set of operands.
Definition: OperandsAndPartials.hpp:38
stan::max_size
size_t max_size(const T1 &x1, const T2 &x2)
Definition: max_size.hpp:9
VectorBuilder.hpp
stan::math::max
int max(const std::vector< int > &x)
Returns the maximum coefficient in the specified column vector.
Definition: max.hpp:21
partials_return_type.hpp
OperandsAndPartials.hpp
length.hpp
stan::VectorBuilder
VectorBuilder allocates type T1 values to be used as intermediate values.
Definition: VectorBuilder.hpp:28
stan::math::size
int size(const std::vector< T > &x)
Return the size of the specified standard vector.
Definition: size.hpp:17
stan::math::check_consistent_sizes
bool check_consistent_sizes(const char *function, const char *name1, const T1 &x1, const char *name2, const T2 &x2)
Return true if the dimension of x1 is consistent with x2.
Definition: check_consistent_sizes.hpp:31
inc_beta.hpp
value_of.hpp
stan::VectorView
VectorView is a template expression that is constructed with a container or scalar, which it then allows to be used as an array using operator[].
Definition: VectorView.hpp:48
check_consistent_sizes.hpp
stan::math::check_positive_finite
bool check_positive_finite(const char *function, const char *name, const T_y &y)
Return true if y is positive and finite.
Definition: check_positive_finite.hpp:28
stan::math::OperandsAndPartials::d_x1
VectorView< T_return_type, false, true > d_x1
Definition: OperandsAndPartials.hpp:39
stan::math::digamma
fvar< T > digamma(const fvar< T > &x)
Definition: digamma.hpp:16
inc_beta_dda.hpp

[ Stan Home Page ] © 2011–2016, Stan Development Team.